//
//  PQBridgeObject.m
//  PQSpeed
//
//  Created by SanW on 2020/8/20.
//  Copyright © 2020 BytesFlow. All rights reserved.
//

#import "PQBridgeObject.h"
#import "CommonCrypto/CommonDigest.h"
#import "CommonCrypto/CommonHMAC.h"
int32_t const CHUNK_SIZE = 8 * 1024;

#define CC_MD5_DIGEST_LENGTH    16          /* digest length in bytes */
#define CC_MD5_BLOCK_BYTES      64          /* block size in bytes */
#define CC_MD5_BLOCK_LONG       (CC_MD5_BLOCK_BYTES / sizeof(CC_LONG))


@implementation PQBridgeObject
+ (NSString *)getByteRate {
    long long intcurrentBytes = [PQBridgeObject getInterfaceBytes];
    NSString *rateStr = [PQBridgeObject formatNetWork:intcurrentBytes];
    return rateStr;
}

+ (long long) getInterfaceBytes {
    struct ifaddrs *ifa_list = 0, *ifa;
    if (getifaddrs(&ifa_list) == -1) {
        return 0;
    }
    uint32_t iBytes = 0;
    uint32_t oBytes = 0;
    for (ifa = ifa_list; ifa; ifa = ifa->ifa_next) {
        if (AF_LINK != ifa->ifa_addr->sa_family)
            continue;
        if (!(ifa->ifa_flags & IFF_UP) && !(ifa->ifa_flags & IFF_RUNNING))
            continue;
        if (ifa->ifa_data == 0)
            continue;
        /* Not a loopback device. */
        if (strncmp(ifa->ifa_name, "lo", 2)){
            struct if_data *if_data = (struct if_data *)ifa->ifa_data;
            iBytes += if_data->ifi_ibytes;
            
            oBytes += if_data->ifi_obytes;
        }
    }
    freeifaddrs(ifa_list);
    NSLog(@"\n[getInterfaceBytes-Total]%d,%d",iBytes,oBytes);
    return (iBytes + oBytes);
}

+ (NSString *)formatNetWork:(long long int)rate {
    if (rate < 1024) {
        return [NSString stringWithFormat:@"%lldB/s", rate];
    } else if (rate >= 1024 && rate < 1024 * 1024) {
        return [NSString stringWithFormat:@"%.1fK/s", (double)rate /1024];
    } else if (rate >= 1024 * 1024 && rate < 1024 * 1024 * 1024) {
        return [NSString stringWithFormat:@"%.1fM/s", (double)rate / (1024*1024)];
    } else {
        return @"0B/s";
    };
}

+ (NSString*)base64ForData:(uint8_t *)input length:(int32_t)length {
    if (input == nil) {
        return nil;
    }
    NSData * data = [NSData dataWithBytes:input length:length];
    return [data base64EncodedStringWithOptions: NSDataBase64Encoding64CharacterLineLength];
}


+ (NSString *)base64Md5ForFilePath:(NSString *)filePath {
    uint8_t * bytes = (uint8_t *)[[self fileMD5:filePath] bytes];
    return [self base64ForData:bytes length:CC_MD5_DIGEST_LENGTH];
}

+ (NSData *)fileMD5:(NSString*)path {
    NSFileHandle *handle = [NSFileHandle fileHandleForReadingAtPath:path];
    if(handle == nil) {
        return nil;
    }
    CC_MD5_CTX md5;
    CC_MD5_Init(&md5);
    BOOL done = NO;
    while(!done) {
        @autoreleasepool{
            NSData* fileData = [handle readDataOfLength: CHUNK_SIZE];
            CC_MD5_Update(&md5, [fileData bytes], (CC_LONG)[fileData length]);
            if([fileData length] == 0) {
                done = YES;
            }
        }
    }
    unsigned char digestResult[CC_MD5_DIGEST_LENGTH * sizeof(unsigned char)];
    CC_MD5_Final(digestResult, &md5);
    return [NSData dataWithBytes:(const void *)digestResult length:CC_MD5_DIGEST_LENGTH * sizeof(unsigned char)];
}

@end
